/*	nkfs for Linux
 *	
 *	Copyright (c) 1995, 1999  Caldera Systems, Inc.
 *	
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *	
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *	
 *	You should have received a copy of the GNU General Public License
 *	along with this program; see the file COPYING.  If not, write to
 *	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *	Boston, MA  02111-1307, USA.
 *
 * 	Please submit patches to nkfs@calderasystems.com
 *
 */

#ifndef NKFS_H
#define NKFS_H

#include <nkfs_rpc.h>

/*	The following definitions are used by the NKFSD (Non-Kernel File
 *	System Driver) to support RPC.
 */

#define NKFS_DRIVER_SIG		0x4E4B4653
#define NKFS_DRIVER_NAME	"nkfs_driver"

typedef struct nkfs_rqstq {
	struct nkfs_rqstq	*next;
	struct nkfs_rqstq	*prev;
	nkfs_rqst_t		*rqst;
	int			free;
}	nkfs_rqstq_t;

typedef struct nkfs_rplyq {
	struct nkfs_rplyq	*next;
	struct nkfs_rplyq	*prev;
	unsigned long		seqnum;
	struct wait_queue	*wait;
	nkfs_rply_t		*rply;
}	nkfs_rplyq_t;

typedef struct nkfs_driver {
	unsigned long		signature;
	unsigned long		state;
#define NKFS_DRIVER_NORMAL	0x00000001
#define NKFS_DRIVER_DRAINING	0x00000002

	unsigned long		seqnum;
	nkfs_rplyq_t		*incoming;
	nkfs_rqstq_t		*outgoing;
	void			(*callback)(void *);
	void			*arg;
	struct wait_queue	*wait;
}	nkfs_driver_t;

/*	The following define nodes in the NKFS File System.
 */

#define NKFS_SUPER_MAGIC	0x24F524F5

typedef struct nkfs_super_block {
	nkfs_driver_t		*driver;
	struct file		*fp;
	struct super_block	*sb;
}	nkfs_super_block_t;

typedef struct nkfs_inode {
	unsigned long		nodeid;
	struct inode		*inode;
	nkfs_super_block_t	*nkfssb;
}	nkfs_inode_t;

#define NKFS_READDIR_SIG	0xABCDEFAB

typedef struct nkfs_readdir_state {
	unsigned long		signature;
	unsigned char		*next;
	nkfs_readdir_rply_t	*reply;
	void			*buffer;
}	nkfs_readdir_state_t;

#endif
