/*	nkfs for Linux
 *	
 *	Copyright (c) 1995, 1999  Caldera Systems, Inc.
 *	
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *	
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *	
 *	You should have received a copy of the GNU General Public License
 *	along with this program; see the file COPYING.  If not, write to
 *	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *	Boston, MA  02111-1307, USA.
 *
 * 	Please submit patches to nkfs@calderasystems.com
 *
 */

#ifndef NKFS_RPC_H
#define NKFS_RPC_H

#ifdef __KERNEL__
#include <linux/types.h>
#else
#include <sys/types.h>
typedef unsigned short	umode_t;
#endif

#define NKFS_NOTIFY_CHANGE	0x00000001
#define NKFS_PUT_INODE		0x00000002
#define NKFS_PUT_SUPER		0x00000003
#define NKFS_STATFS		0x00000004
#define NKFS_CREATE		0x00000005
#define NKFS_LOOKUP		0x00000006
#define NKFS_UNLINK		0x00000007
#define NKFS_MKDIR		0x00000008
#define NKFS_RMDIR		0x00000009
#define NKFS_RENAME		0x0000000A
#define NKFS_TRUNCATE		0x0000000B
#define NKFS_PERMISSION		0x0000000C
#define NKFS_READ		0x0000000D
#define NKFS_WRITE		0x0000000E
#define NKFS_READDIR		0x0000000F
#define NKFS_OPEN		0x00000010
#define NKFS_RELEASE		0x00000011
#define NKFS_READLINK		0x00000012
#define NKFS_FOLLOWLINK		0x00000013
#define NKFS_UNLOCKNODE		0x00000014

typedef struct nkfs_rqst_hdr {
	unsigned long	msglen;
	unsigned long	msgtype;
	unsigned long	seqnum;
	uid_t		uid;
	gid_t		gid;
}	nkfs_rqst_hdr_t;

typedef struct nkfs_rply_hdr {
	unsigned long	msglen;
	unsigned long	msgtype;
	unsigned long	seqnum;
	long		result;
}	nkfs_rply_hdr_t;
	
typedef struct nkfs_rqst {
	nkfs_rqst_hdr_t	hdr;
	unsigned char	data[0];
}	nkfs_rqst_t;

typedef struct nkfs_rply {
	nkfs_rply_hdr_t	hdr;
	unsigned char	data[0];
}	nkfs_rply_t;

typedef struct nkfs_node_info {
	unsigned long	nodeid;
	unsigned long	ino;
	umode_t		mode;
	nlink_t		nlink;
	uid_t		uid;
	gid_t		gid;
	off_t		size;
	time_t		atime;
	time_t		mtime;
	time_t		ctime;
}	nkfs_node_info_t;

typedef struct nkfs_notify_chg_rqst {
	unsigned long	nodeid;
	umode_t		mode;
	uid_t		uid;
	gid_t		gid;
	off_t		size;
	time_t		atime;
	time_t		mtime;
}	nkfs_notify_chg_rqst_t;

typedef struct nkfs_notify_chg_rply {
	nkfs_node_info_t	nodeinfo;
}	nkfs_notify_chg_rply_t;

typedef struct nkfs_put_inode_rqst {
	unsigned long	nodeid;
}	nkfs_put_inode_rqst_t;

typedef struct nkfs_statfs_rply {
	long	bavail;
	long	bfree;
	long	bsize;
	long	blocks;
	long	files;
	long	ffree;
	long	namelen;
}	nkfs_statfs_rply_t;
	
typedef struct nkfs_create_rqst {
	unsigned long	nodeid;
	int		mode;
	char		name[255];
}	nkfs_create_rqst_t;

typedef struct nkfs_create_rply {
	nkfs_node_info_t	nodeinfo;
}	nkfs_create_rply_t;

typedef struct nkfs_lookup_rqst {
	unsigned long	nodeid;
	char		name[255];
}	nkfs_lookup_rqst_t;

typedef struct nkfs_lookup_rply {
	nkfs_node_info_t	nodeinfo;
}	nkfs_lookup_rply_t;

typedef struct nkfs_unlink_rqst {
	unsigned long	nodeid;
	char		name[255];
}	nkfs_unlink_rqst_t;

typedef struct nkfs_mkdir_rqst {
	unsigned long	nodeid;
	int		mode;
	char		name[255];
}	nkfs_mkdir_rqst_t;

typedef struct nkfs_rmdir_rqst {
	unsigned long	nodeid;
	char		name[255];
}	nkfs_rmdir_rqst_t;

typedef struct nkfs_rename_rqst {
	unsigned long	onodeid;
	unsigned long	nnodeid;
	char		oname[255];
	char		nname[255];
}	nkfs_rename_rqst_t;

typedef struct nkfs_truncate_rqst {
	unsigned long	nodeid;
	unsigned long	length;
}	nkfs_truncate_rqst_t;

typedef struct nkfs_permission_rqst {
	unsigned long	nodeid;
	unsigned long	flags;
}	nkfs_permission_rqst_t;

#define NKFS_MAX_READ	(3*1024)

typedef struct nkfs_read_rqst {
	unsigned long	nodeid;
	off_t		offset;
	int		count;
}	nkfs_read_rqst_t;

typedef struct nkfs_read_rply {
	int		count;
	unsigned char	data[0];
}	nkfs_read_rply_t;

typedef struct nkfs_readlink_rqst {
	unsigned long	nodeid;
}	nkfs_readlink_rqst_t;

typedef struct nkfs_readlink_rply {
	int		count;
	unsigned char	data[0];
}	nkfs_readlink_rply_t;

#define NKFS_MAX_WRITE	(3*1024)

typedef struct nkfs_write_rqst {
	unsigned long	nodeid;
	off_t		offset;
	int		count;
	unsigned char	data[0];		
}	nkfs_write_rqst_t;

typedef struct nkfs_write_rply {
	int			count;
	nkfs_node_info_t	nodeinfo;
}	nkfs_write_rply_t;

typedef struct nkfs_dirent {
	unsigned long	ino;
	long		entrylen;
	char		name[0];
}	nkfs_dirent_t;

#define NKFS_READDIR_START	0x00000001
#define NKFS_READDIR_MORE	0x00000002
#define NKFS_READDIR_FINISH	0x00000003

typedef struct nkfs_readdir_rqst {
	unsigned long	nodeid;
	unsigned long	handle;
	unsigned long	verb;
}	nkfs_readdir_rqst_t;

typedef struct nkfs_readdir_rply {
	unsigned long	handle;
	unsigned long	count;
	unsigned char	buffer[0];
}	nkfs_readdir_rply_t;
	
#define NKFS_READ_ACCESS	0x00000001
#define NKFS_WRITE_ACCESS	0x00000002
#define NKFS_EXEC_ACCESS	0x00000004

typedef struct nkfs_open_rqst {
	unsigned long	nodeid;
	unsigned long	flags;
}	nkfs_open_rqst_t;

typedef struct nkfs_release_rqst {
	unsigned long	nodeid;
	unsigned long	handle;
}	nkfs_release_rqst_t;

typedef struct nkfs_unlock_rqst {
	unsigned long	nodeid;
}	nkfs_unlock_rqst_t;
#endif

